<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: default_config.php 17927 2009-02-16 13:16:09Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

# ----------------- Features ----------------------------------------------

# Whether the firewall is NuFW or plain netfilter
# (see "Netfilter and NuFW options" above)
$nufw_firewall = true;

# Whether the firewall has Layer 7 filtering capabilities
$l7_firewall = false;

# Support transparent proxy option
$use_transparent_proxy = false;

# Propagate firewall on other hosts using $sync_script shell script
# (see $sync_* variables above for the details)
$sync_enabled = false;

# ----------------- Debug options -----------------------------------------

# Enable debug mode
$debug_nuface = false;

# Display more informations in debug mode (eg. traceback)
$debug_nuface_verbose = false;

# ----------------- Language options --------------------------------------

# Default language of the interface
$default_language = 'en';

# ----------------- Webpage options ---------------------------------------

# Display image as logo
$logo_nuface = "images/nupik.png";

# Display logout link?
$logout_link = false;

# Use animation-min.js from libyahoo?
$context_help = true;

# EdenWall integration:
# load the edenavbar for integration with NuConf, NuLog, etc.
$edenwall_integration = false;

# allow to manage the local interface with nuface
$manage_local_firewall = true;

# ----------------- Netfilter and NuFW options ----------------------------

# Apply firewall rules? (load init-firewall)
$apply_firewall_rules = true;

# Time period (periodicity and duration): requires Netfilter
# FIXED_TIMEOUT option (included in Linux kernel 2.6.14+)
$support_periods = true;

# Whether the firewall supports source port SNAT randomization
# (included in Linux kernel 2.6.21 or greater)
$netfilter_snat_supports_sport_randomization = false;

# Valid limits for log used with --limit
$NO_LOG = 'no';
$DEFAULT_LOG = 'unlimited';

# ----------------- Layer seven (L7) options ------------------------------

$l7_mask = "0x007f0000";
$l7_command = "/usr/sbin/nulayer7";
$l7_protos_file = "include/l7-protos";

# ----------------- Nupyf options -----------------------------------------

# Add --no-check-net command line option
$disable_check_net = false;

# Add --same-iface: Generate unused bichains with nupyf
# Always enabled if desc.xml only contains one interface
$nupyf_same_iface = true;

# Use ULOG Netfilter target for packet logging (instead of LOG)?
# Always true if $nufw_firewall=true
$nupyf_use_ulog = false;

# ----------------- Directories and files ---------------------------------

$tmpdir = "/tmp";
$etc_dir = "/etc/nuface";
$locale_dir = "/usr/share/locale";
$nuface_dir = "/var/lib/nuface";
$datadir = "$nuface_dir/acls";
$logdir = "$nuface_dir/logs";
$lockdir = $tmpdir;
$rules_dir = "$nuface_dir/dyn";
$nufw_rules_dir = "$nuface_dir/dyn/nufw";
$std_rules_dir = "$nuface_dir/dyn/standard";
$network_desc_dir = "$nuface_dir/desc";

$nupyf_ldap_file = "ldap_objects";
$nupyf_conf_file = "nupyf.conf";
$periods_file = "$tmpdir/periods.xml";
$network_desc_file = "desc.xml";

# ----------------- Scripts -----------------------------------------------

$sudo_command = "/usr/bin/sudo";
$nupyf_command = "/usr/sbin/nupyf";
$nuperiod_command = "/usr/sbin/nuperiod";
$gendesc_script = "/usr/sbin/gendesc";
$checkdesc_script = "/usr/sbin/checkdesc";
$firewall_script = "/etc/init.d/init-firewall";

# ----------------- Group options -----------------------------------------

# Wheter to use getent to retrieve groups information: name and gid
$use_getent_groups = true;

# Whether to decode UTF8 (if conversion is needed)
$decode_groups_from_utf8 = false;

# Path to file (in /etc/group format) to use instead of system conf
# (use_getent_groups must be set to true)
# $groups_file='/tmp/groups.txt';

# Path to file representing local groups, typicaly /etc/groups
# Use it if local groups must not be displayed in nufw elts groups lists
# $sys_groupfile = '/etc/group';

# ----------------- Duplicate firewall options ----------------------------

# Global settings are used if customs settings are not specified on each host
$sync_user = 'nuface';
$sync_script = '/usr/sbin/sync_firewalls.sh';
$sync_remote_rules_dir = $rules_dir;
$sync_remote_sudo_command = $sudo_command;
$sync_remote_firewall_script = $firewall_script;
$sync_stop_apply_on_fail = true;

# Description of all hosts. Only "host" key is mandatory,
# otherwise $sync_* variables will be used. Example:
#
#    $sync_hosts = Array(
#        Array(
#            'hostname' => 'firewall-slave1',
#            'user' => 'nuface',
#            'remote_rules_dir' => $rules_dir,
#            'remote_sudo_command' =>  $sudo_command,
#            'remote_firewall_script' => $firewall_script,
#        ),
#        Array(
#            'hostname' => 'firewall-slave2',
#            'user' => 'root',
#            'remote_rules_dir' => "/opt/nuface/rules",
#            'remote_sudo_command' =>  "/usr/local/sbin/sudo",
#            'remote_firewall_script' => "/opt/nuface/init-firewall",
#        ),
#    );

# ----------------- Transparent proxy -------------------------------------

$transparent_proxy_port = 3128;

# -------------------------------------------------------------------------
?>
