<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: elt_set.class.php 17927 2009-02-16 13:16:09Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('set.class.php');    // set class

function xml_read_elements(&$object, $parent_node, $tag_name="elt", $class="elt")
{
    foreach($parent_node->get_elements_by_tagname($tag_name) as $node)
    {
        $item = new $class($node);
        $object->add_elt($item);
    }
}

$ELT_SET_ATTRIBUTES = Array(
    'name', 'comment', 'modified',
    'defaultaction', 'defaultprefix',
);

$L7RULE_DEFAULT_ACTIONS = Array(
    'drop', 'accept',
    'ulogaccept', 'ulogdrop',
    'logaccept', 'logdrop',
);

class elt_set extends set
{
    var $name;
    var $ID;

    function elt_set($ress, $type='xml', $id=0, $element_type=null, $ruleset=null)
    {
        $this->elts = Array();
        $this->element_type = $element_type;
        if ($type != 'xml') {
            $this->ID=$id;
            if (array_key_exists('defaultaction', $ress))
            {
                $this->defaultaction=$ress['defaultaction'];
            }
            if (array_key_exists('defaultprefix', $ress))
            {
                $this->defaultprefix=$ress['defaultprefix'];
            }
            $this->name=$ress['name'];
            if (array_key_exists('comment',$ress))
                $this->comment=$ress['comment'];
            else
                $this->comment='';
        }else{
            $dfltac=$ress->get_attribute('defaultaction');
            if ((isset($dfltac)) and ($dfltac!='--') and ($dfltac!=''))
            {
                $this->defaultaction=$ress->get_attribute('defaultaction');
            }
            $dfltpre=$ress->get_attribute('defaultprefix');
            if (isset($dfltpre) and ($dfltpre!=''))
            {
                $this->defaultprefix=$ress->get_attribute('defaultprefix');
            }
            $this->name=$ress->get_attribute('name');
            $this->ID = $ress->get_attribute('ID');
            $this->comment = $ress->get_attribute('comment');
            if (!$this->ID) {
                $this->ID = $ress->get_attribute('id');
            }
        }

        # Set value of $this->defaultaction if it's not set
        # Hack needed for ruleset created on nuface older than version 2.0.11
        if ($element_type == 'l7rule'
            && (!property_exists($this, 'defaultaction') || $this->defaultaction == ''))
        {
            $this->defaultaction = 'drop';
            add_log(sprintf(
                _("Set default action of %s to %s."),
                $this->str(), $this->defaultaction));
        }

        if (!property_exists($this, 'modified')) {
            $this->modified = modifiedTimestamp();
        }

        $this->check_throw_consistency($ruleset);
    }

    function xmldump($xml,$node)
    {
        $node->set_attribute('name',$this->name);
        if ((isset($this->defaultaction)) and ($this->defaultaction!='--') and ($this->defaultaction!=''))
        {
          $node->set_attribute('defaultaction',$this->defaultaction);
        }
        if (isset($this->defaultprefix) and ($this->defaultprefix!=''))
        {
          $node->set_attribute('defaultprefix',$this->defaultprefix);
        }
        $node->set_attribute('ID',$this->ID);
        $node->set_attribute('comment',$this->comment);
        if (isset($this->elts))
        {
          foreach ($this->elts as $elt){
                $elt->xmldump($xml,$node);
          }
        }
    }

    function str()
    {
        $type = get_class($this);
        $type_names = Array(
            'protocol' => _('Protocol'),
        );
        if (array_key_exists($type, $type_names))
            $text = $type_names[$type];
        else if ($this->element_type)
            $text = $this->element_type;
        else
            $text = _('Element set');
        if ($this->name)
            $text .= ' "'.$this->name.'"';
        if ($this->ID)
            $text .= ' (#'.$this->ID.')';
        return $text;
    }

    function check_consistency()
    {
        global $L7RULE_DEFAULT_ACTIONS;

        if (!$this->ID)
            return _('Invalid identifier');

        if ($this->element_type == 'l7rule')
        {
            $action = $this->defaultaction;
            if (!in_array($action, $L7RULE_DEFAULT_ACTIONS))
                return sprintf(_('Invalid default action: "%s"'), $action);
        }
        return '';
    }

    function check_throw_consistency($ruleset=null)
    {
        $errmsg = $this->check_consistency($ruleset);
        if ($errmsg)
        {
            $errmsg = sprintf(_('%s consistency error: %s!'), $this->str(), $errmsg);
            throw new Exception($errmsg);
        }
    }

    function modify_param($key,$value){
        global $ELT_SET_ATTRIBUTES;

        if (!in_array($key, $ELT_SET_ATTRIBUTES))
        {
            throw new Exception(sprintf(
                _('Unable to change value of the parameter "%s" of %s.'),
                $key, $this->str()));
        }

        $old_value = $this->$key;
        try {
            $this->$key = $value;
            $this->check_throw_consistency();
        } catch (Exception $err) {
            $this->$key = $old_value;
            throw $err;
        }
    }

    function get_id(){
      return($this->ID);
    }

    function get_max_elt_id(){
        $max=0;
        foreach ($this->elts as $elt){
            $local=$elt->get_id();
            if ($local>$max)
              $max=$local;
        }
        return $max;
    }

    function list_tab()
    {
        $result = Array();
        foreach ($this->elts as $elt){
          if (array_key_exists('name', $elt->datas) and $elt->datas['name']!=''){
            $result{$elt->datas['ID']} = ($elt->datas['name']);
          }else{
            $result{$elt->datas['ID']} = $elt->datas['ID'];
          }
        }
      return $result;
    }

    function get_obj()
    {
      $result = Array();
      foreach ($this as $key=>$obj)
      {
        // Skip elts, that contains stuff from subelements
        if ($key=='elts')
          continue;
        $result{$key}=$obj;
      }
      return $result;
    }

    function read_elements($parent_node, $tag_name="elt")
    {
        xml_read_elements($this, $parent_node, $tag_name);
    }
}

?>
