<?php
# Copyright(C) 2004-2005 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: security.php 17927 2009-02-16 13:16:09Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('types.php');

function check_ip($ip)
{
  if (!preg_match('/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/',$ip,$matches))
    return(0);
  if (($matches[1]<1) or ($matches[1]>255))
    return(0);
  for ($i=2;$i<5;$i++)
    if (($matches[$i]<0) or ($matches[$i]>255))
      return(0);
  return(1);
}

function is_a_net($var, $name)
{
    if (preg_match('/^$/',$var))
        return true;
    if (preg_match('/^(?:\! |)\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(?:|\/\d{1,3}(?:|\.\d{1,3}\.\d{1,3}\.\d{1,3}))$/',$var))
        return true;
    log_error(sprintf(_('Sorry, bad parameter received for "%s": "%s"!'), $name, $var));
    return false;
}

function check_log_prefix($value, $name)
{
    if (!$value)
        return true;
    if (22 < strlen($value)) {
        log_error(sprintf(_('Log prefix (%s) is too long: %s characters whereas maximum is %s!'),
                    $name, strlen($value), 22));
        return false;
    }
    if (preg_match('!^[a-zA-Z0-9 \.\-_]+$!',$value))
        return true;
    log_error(sprintf(_('Log prefix (%s) can only contains letters, digits, space and characters "._-": "%s"!'),
        $name, htmlspecialchars($value)));
    return false;
}

function check_name($value, $name, $verbose=true)
{
    if (preg_match('@^[a-zA-Z0-9 _.,:!()\\\\/-]*$@', $value))
        return true;
    if ($verbose)
        log_error(sprintf(_('Invalid name received for "%s": "%s"!'),
            $name, htmlspecialchars($value)));
    return false;
}

function check_search($name)
{
  if (!isset($name))
    return(1);
  if (!preg_match('/^[\w\.\-\s]*$/',$name))
    return(0);
  return(1);
}

function check_comment($value, $name, $verbose=true)
{
    if (preg_match('/^[a-zA-Z0-9\.\-\_\sàùéèçêï]*$/',$value))
        return true;
    if ($verbose)
        log_error(sprintf(_('Sorry, bad parameter received for "%s": "%s"!'),
            $name, htmlspecialchars($value)));
    return false;
}

function check_nb($nb, $verbose=true)
{
    if (!isset($nb))
        return true;
    if (preg_match('/^[0-9]*$/',$nb))
        return true;
    if ($verbose)
        log_error(sprintf(_('Sorry, bad parameter received: "%s"!'), $nb));
    return false;
}

function check_array_of_nb($tab)
{
  if (!isset($tab))
    return(1);
  foreach ($tab as $nb) {
    if (!preg_match('/^[0-9]*$/',$nb))
      return(0);
  }
  return(1);
}

function is_empty($var)
{
  if (!isset($var))
    return(1);
  if (preg_match('/^$/',$var))
    return(1);
  return(0);
}

function check_filename($filename)
{
  if (!isset($filename))
    return true;
  if (preg_match('/\.\./',$filename))
    return false;
  if (!preg_match('/^[a-zA-Z0-9\.\-_,]*$/',$filename))
    return false;
  return true;
}

function check_decision($decision)
{
    global $VALID_DECISIONS;

    if (!isset($decision))
        return true;
    if (!in_array($decision, $VALID_DECISIONS)) {
        log_error(_('Sorry, cannot work on non-existing element')." no decision $decision");
        return false;
    }
    return true;
}

function check_aclorder($order, $name)
{
    if (!isset($order))
        return true;
    if (preg_match('/^[0-9|]*$/',$order))
        return true;
    if ($name)
        log_error(sprintf(_('Sorry, bad parameter received for "%s": "%s"!'), $name, $order));
    return false;
}

function check_port_range($value)
{
    if ($value == "")
    {
      return true;
    }
    if (preg_match("/^[0-9]+$/", $value)) {
        $start = intval($value);
        $stop = $start;
    } else if (preg_match("/^[0-9]+:[0-9]+$/", $value)) {
        $value = explode(':', $value);
        $start = intval($value[0]);
        $stop = intval($value[1]);
    } else {
        return false;
    }
    if ($stop < $start) {
        return false;
    }
    if ($start < 1) {
        return false;
    }
    if (65535 < $stop) {
        return false;
    }
    return true;
}


function check_protocol($value)
{
    global $VALID_PROTOCOLS;
    return in_array($value, $VALID_PROTOCOLS);
}

$valid_icmp_types = Array('--', 'any',
    'echo-reply', 'destination-unreachable', 'network-unreachable',
    'host-unreachable', 'protocol-unreachable', 'port-unreachable',
    'fragmentation-needed', 'source-route-failed', 'network-unknown',
    'host-unknown', 'network-prohibited', 'host-prohibited',
    'TOS-network-unreachable', 'TOS-host-unreachable',
    'communication-prohibited', 'host-precedence-violation',
    'precedence-cutoff', 'source-quench',
    'redirect', 'network-redirect', 'host-redirect',
    'TOS-network-redirect', 'TOS-host-redirect',
    'echo-request', 'router-advertisement',
    'router-solicitation', 'time-exceeded',
    'ttl-zero-during-transit', 'ttl-zero-during-reassembly',
    'parameter-problem', 'ip-header-bad',
    'required-option-missing', 'timestamp-request',
    'timestamp-reply', 'address-mask-request',
    'address-mask-reply');

function check_icmp_types($value)
{
    global $valid_icmp_types;
    return in_array($value, $valid_icmp_types);
}

function check_element_type($type)
{
    $valid_types = Array(
        // desc.xml types
        'address', 'network', 'connection',
        // acls.xml and nuface types
        'ipv4', 'nufw', 'link', 'l7rule', 'os',
        'proto', 'app', 'periodicity', 'local', 'duration',
        'mark',
    );
    return in_array($type, $valid_types);
}

?>
