<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: acl.class.php 17927 2009-02-16 13:16:09Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('aclbase.class.php');
require_once('nuphp/session.php');

define('NO_PROXY', 0);
define('USE_PROXY', 1);
define('OVERRIDE_PROXY', 2);

class acl extends aclbase
{
    var $with;
    var $on;
    var $auth;
    var $periodicity;
    var $duration;
    var $transparent_proxy;

    function acl(&$ruleset, $ress, $type='xml')
    {
        parent::aclbase('FORWARD');

        try {
            if ($type != 'xml') {
                $this->loadArray($ress);
            }else{
                $this->loadXML($ress);
            }
        } catch (Exception $err) {
            throw new Exception(sprintf(
                        _("Error when reading %s: %s"),
                        $this->str(), $err->getMessage()));
        }

        $errmsg = $this->check_consistency($ruleset);
        if ($errmsg)
        {
            $errmsg = sprintf(_("%s consistency error: %s!"), $this->str(), $errmsg);
            throw new Exception($errmsg);
        }
    }

    function check_consistency($ruleset)
    {
        $errmsg = parent::check_consistency($ruleset);
        if ($errmsg) {
            return $errmsg;
        }

        if ($this->auth
        and !check_proto_can_get_auth($ruleset->protocols, $this->proto)) {
            return _('The chosen set of protocols cannot be authenticated by NuFW');
        }

        if ($this->transparent_proxy == USE_PROXY and !$this->auth) {
            return _('You can not use the transparent proxy without being authenticated by NuFW');
        }

        if ($this->transparent_proxy == OVERRIDE_PROXY and $this->auth) {
            return _('You can not override the transparent proxy and being authenticated by NuFW');
        }

        if ($this->periodicity and $this->duration) {
            return _('An ACL cannot be assigned a duration together with a periodicity');
        }

        if ($this->periodicity
        and !$ruleset->periodicities->has_elt($this->periodicity)) {
            return sprintf(_('Broken reference of attribute %s=%s'), 'periodicity', $this->periodicity);
        }

        if ($this->duration and !$ruleset->durations->has_elt($this->duration)) {
            return sprintf(_('Broken reference of attribute %s=%s'), 'duration', $this->duration);
        }

        if ($this->with and !$ruleset->applications->has_elt($this->with)) {
            return sprintf(_('Broken reference of attribute %s=%s'), 'with', $this->application);
        }

        if ($this->auth and !$ruleset->auths->has_elt($this->auth)) {
            return sprintf(_('Broken reference of attribute %s=%s'), 'auth', $this->auth);
        }

        if ($this->on and !$ruleset->oss->has_elt($this->on)) {
            return sprintf(_('Broken reference of attribute %s=%s'), 'on', $this->on);
        }
        return '';
    }

    function loadArray($ress)
    {
        parent::loadArray($ress);
        $this->auth = array_get($ress, 'auth');
        $this->periodicity = array_get($ress, 'periodicity');
        $this->duration = array_get($ress, 'duration');
        $this->transparent_proxy = array_get($ress, 'transparent_proxy', NO_PROXY);
        $this->with = array_get($ress, 'with');
        $this->on = array_get($ress, 'on');
        if (array_key_exists('descsorts', $ress))
            $this->descsorts = $ress['descsorts'];
        else
            $this->descsorts = new obj_set('descsorts', Array(), false);
    }

    function loadXML($dom)
    {
        global $NO_LOG;

        parent::loadXML($dom);

        $this->auth = xmlGetAttribute($dom, 'auth');
        $this->periodicity = xmlGetAttribute($dom, 'periodicity');
        $this->duration = xmlGetAttribute($dom, 'duration');
        $this->with = xmlGetAttribute($dom, 'with');
        $this->on = xmlGetAttribute($dom, 'on');
        $this->transparent_proxy = (int)xmlGetAttribute($dom, 'transparent_proxy', NO_PROXY);

        $descsorts = $dom->get_elements_by_tagname("descsorts");
        /* iter on resource */
        $hash_ds=array();
        if (count($descsorts)==1)
        {
            $descsorts = $descsorts[0];
            $descsort_list = $descsorts->get_elements_by_tagname("descsort");
            foreach ($descsort_list as $ds){
                $newds=new descsort($ds);
                $id = $ds->get_attribute("ID");
                $hash_ds[$id] = $newds;
            }
        }
        $this->descsorts=new obj_set('descsorts', $hash_ds, false);
    }

    function xmldump($xml,$node)
    {
        parent::writeXML($node);
        $node->set_attribute('auth',$this->auth);
        if ($this->with)
          $node->set_attribute('with',$this->with);
        if ($this->on)
          $node->set_attribute('on',$this->on);
        if ($this->periodicity)
          $node->set_attribute('periodicity',$this->periodicity);
        if ($this->duration)
          $node->set_attribute('duration',$this->duration);
        if ($this->transparent_proxy)
          $node->set_attribute('transparent_proxy', $this->transparent_proxy);
        $descnode = $xml->create_element("descsorts");
        $node->append_child($descnode);
        $descsortsobjets=$this->descsorts->elts;
        if (is_array($descsortsobjets))
        {
          foreach($descsortsobjets as $descsortsobjet){
            $descsortsobjet->xmldump($xml,$descnode);
          }
        }

    }

    function new_id(){
      return($this->get_max_id()+1);
    }
    function get_id(){
      return($this->ID);
    }
    function get_max_elt_order($from=-2,$to=-2){
        $max=0;
        if (($from == -2) or ($to == -2))
        {
          return(-1);
        }
        if ((isset($this->descsorts) and (is_array($this->descsorts->elts))))
        {
          foreach ($this->descsorts->elts as $elt){
            if(isset($elt) && $elt != null && $elt->elts != null){
              $local=$elt->get_order($from,$to);
              if ($local>$max)
                $max=$local;
            }
          }
        }
        return $max;
    }

    function get_max_elt_order_id(){
        $max=0;
        if (isset($this->descsorts) and (is_array($this->descsorts->elts)))
        {
          foreach ($this->descsorts->elts as $elt){
              if(isset($elt) && $elt != null && $elt->elts != null){
              $local=$elt->get_max_id();
              if ($local>$max)
                $max=$local;
              }
          }
        }
        return $max;
    }

    #
    # get acl order in desc referenced by desc_id
    # for bichain referenced by (from,to)
    function get_order($desc_id, $from, $to){
        $order = 0;
        foreach ($this->descsorts->elts as $elt){
            if ($elt->ID == $desc_id){
                $order = $elt->get_order($from, $to);
                if($order != 0){
                    return $order;
                }
            }
        }
        return $order;
    }

    #
    # set acl order in desc referenced by desc_id
    # for bichain referenced by (from,to)
    # with order
    function set_order($desc_id, $from, $to, $order, $new_id){
        if (isset($this->descsorts))
	{
        	if ($this->descsorts->elts == null){
            		$this->descsorts->elts = array();
        	}
    	} else {
            $this->descsorts = new obj_set('descsorts', Array(), false);
        }
        if (!isset($this->descsorts->elts[$desc_id]) || $this->descsorts->elts[$desc_id] == null || $this->descsorts->elts[$desc_id]->elts == null )
        {
            $newdesc = new descsort(array('elts' => null),'data',$desc_id);
            $this->descsorts->replace_elt($newdesc);
        } else {
            $newdesc =& $this->descsorts->elts[$desc_id];
        }
        foreach ($this->descsorts->elts as $elt){
            if ($elt->ID == $desc_id){
                $elt_order = $elt->get_elt($from, $to);
                if (!isset($elt_order))
                {
                  $creator['order']=$order;
                  $creator['from']=$from;
                  $creator['to']=$to;
                  $new_order = new order_elt($creator,'data',$new_id);
                  $newdesc->add_elt($new_order);
                }
            }
          }
        foreach ($this->descsorts->elts as $elt){
            if ($elt->ID == $desc_id){
                $elt_order = $elt->get_elt($from, $to);
                $elt_order->modify_param('order', $order);
                $newdesc->replace_elt($elt_order);
                return;
            }
        }
    }

    function cloneobj(&$document){
      $t=array();
      $t['name'] = $this->name;
      $t['proto'] = $this->proto;
      $t['periodicity'] = $this->periodicity;
      $t['duration']= $this->duration;
      $t['from'] = $this->from;
      $t['auth'] = $this->auth;
      $t['with'] = $this->with;
      $t['to'] = $this->to;
      $t['ID'] = $this->ID;
      $t['decision'] = $this->decision;
      $t['log'] = $this->log;
      $t['log_prefix'] = $this->log_prefix;
      $t['modified'] = $this->modified;
      $t['comment'] = $this->comment;
      $t['group'] = $this->group;
      $t['transparent_proxy'] = $this->transparent_proxy;
      return new acl($document, $t, $type='toto');
    }

    function get_descsort()
    {
        $used_desc = getSession('used_desc', false);
        $descsorts = $this->descsorts->elts;
        if (array_key_exists($used_desc, $descsorts))
            return $descsorts[$used_desc];
        else
            return null;

    }
}

?>
